/'****************************************************************************
*
* Name: fbstack.bi
*
* Synopsis: A stack object for FB programs.
*
* Description: This program implements a stack object for use in FB programs.
*
* Copyright 2009, Richard D. Clark
*
*                          The Wide Open License (WOL)
*
* Permission to use, copy, modify, distribute and sell this software and its
* documentation for any purpose is hereby granted without fee, provided that
* the above copyright notice and this license appear in all source copies. 
* THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY OF
* ANY KIND. See http://www.dspguru.com/wol.htm for more information.
*
*****************************************************************************'/

'Put the stack within a namespace.
Namespace fbStackarr
	
'Include the standard data object.
#Include "dataobj.bi"

'Define True/False values.
#Ifndef FALSE
	#Define FALSE 0
   #Define TRUE (Not FALSE)
#EndIf

'Stack error codes.
Enum stackerror
	stacknoerror = 0
	stackempty
	stackfull
End Enum

'Stack object definiton.
Type stackobj
	Private:
	_stack (0 To 10) As dataobj 'The actual stack. The 0 index is used as the empty flag. 
	_cnt As Integer 'The number of items in the stack.
	_lasterror As stackerror 'The last error.
	Declare Sub _ClearStack() 'Clears the stack, setting each element to NULL.
	Public:
	Declare Constructor ()
	Declare Destructor ()
	Declare Sub ClearStack () 'Calls _ClearStack.
	Declare Function Count() As Integer 'Number of items on the stack.
	Declare Function StackSize () As Integer 'Returns the size of the stack, i.e. the upper bounds.
	Declare Function Push (dt As dataobj) As Integer 'Pushes an item onto stack. Returns True/False.
	Declare function Pop (dt As dataobj) As Integer 'Retrieves a data obj from stack. Returns True/False.
	Declare Function GetLastError () As stackerror 'Returns the last error code.
	Declare Function GetErrorMessage (e As stackerror) As String 'Returns the error as a string. 
End Type

'Deallocate the stack.
Sub stackobj._ClearStack ()
	'Clear each data object in stack.
	For i As Integer = 0 To UBound(_stack) 
		_stack(i).ClearData
	Next
	'Set the number of items to zero.
	_cnt = 0
End Sub

'Constructor
Constructor stackobj ()
	'Clear stack data.
	_ClearStack
	'Set default error.
	_lasterror = stacknoerror
End Constructor

'Destructor
Destructor stackobj ()
	_ClearStack
End Destructor

'Clears stack. 
Sub stackobj.ClearStack ()
	_ClearStack
End Sub

'Returns the number of items on the stack or 0 if empty.
Function stackobj.Count () As Integer
	Return _cnt
End Function

'Returns the size of the stack, i.e. the upper bounds. Lower bound is always 1.
Function stackobj.StackSize () As Integer
	Return UBound(_stack)
End Function

'Pushes an item onto the stack. Returns True if successful, False if an error occured.
Function stackobj.Push (dt As dataobj) As Integer
	Dim ret As Integer = TRUE
	
	'Set the default error code.
	_lasterror = stacknoerror
	'Is the stack full?
	If _cnt = UBound(_stack) Then
		ret = FALSE
		_lasterror = stackfull
	Else
		'Add the new element to the stack.
		_cnt += 1
		_stack(_cnt) = dt
	EndIf
	
	Return ret
End Function

'Pops an item from the stack. If stack is empty returns False, else True is returned.
Function stackobj.Pop (dt As dataobj) As Integer
	Dim ret As Integer = TRUE
	
	'Set the default error code.
	_lasterror = stacknoerror
	'Nothing on the stack.
	If _cnt = 0 Then
		ret = FALSE
		_lasterror = stackempty
	Else
		'Return the top item.
		dt = _stack(_cnt)
		'Set the item to NULL.
		_stack(_cnt).ClearData
		_cnt -= 1
	EndIf
	
	Return ret
	
End Function

'Returns the last error code.
Function stackobj.GetLastError () As stackerror
	Return _lasterror
End Function

'Returns the error as a string.
Function stackobj.GetErrorMessage (e As stackerror) As String
	Dim ret As String
	
	Select Case e
		Case stacknoerror
			ret = "Stack: No error"
		Case stackempty
			ret = "Stack: Stack empty"
		Case stackfull
			ret = "Stack: Stack full"
		Case Else
			ret = "Stack: Unknown error code"
	End Select
	
	Return ret
End Function

End Namespace
