/'****************************************************************************
*
* Name: fbstack.bi
*
* Synopsis: A stack object for FB programs.
*
* Description: This program implements a stack object for use in FB programs.
*
* Copyright 2009, Richard D. Clark
*
*                          The Wide Open License (WOL)
*
* Permission to use, copy, modify, distribute and sell this software and its
* documentation for any purpose is hereby granted without fee, provided that
* the above copyright notice and this license appear in all source copies. 
* THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY OF
* ANY KIND. See http://www.dspguru.com/wol.htm for more information.
*
*****************************************************************************'/

'Put the stack within a namespace.
Namespace fbStack
	
'Include the standard data object.
#Include "dataobj.bi"

'Define True/False values.
#Ifndef FALSE
	#Define FALSE 0
   #Define TRUE (Not FALSE)
#EndIf

'Stack error codes.
Enum stackerror
	stacknoerror = 0
	stackempty
	stacknomemory
End Enum

'Stack object definiton.
Type stackobj
	Private:
	_stack As dataobj Ptr 'The actual stack.
	_cnt As Integer 'The number of items in the stack.
	_lasterror As stackerror 'The last error.
	Declare Sub _ClearStack() 'Deallocates the stack.
	Public:
	Declare Constructor ()
	Declare Destructor ()
	Declare Sub ClearStack () 'Clears stack.
	Declare Function Count() As Integer 'Number of items on the stack.
	Declare Function Push (dt As dataobj) As Integer 'Pushes an item onto stack. Returns True/False.
	Declare function Pop (dt As dataobj) As Integer 'Retrieves a data obj from stack. Returns True/False.
	Declare Function GetLastError () As stackerror 'Returns the last error code.
	Declare Function GetErrorMessage (e As stackerror) As String 
End Type

'Deallocate the stack.
Sub stackobj._ClearStack ()
	If _stack <> NULL Then
		'Clear each data object in stack.
		For i As Integer = 0 To _cnt - 1 
			_stack[i].ClearData
		Next
		'Deallocate the stack.
		DeAllocate _stack
	EndIf
End Sub

'Constructor
Constructor stackobj ()
	_stack = NULL
	_cnt = 0
	_lasterror = stacknoerror
End Constructor

'Destructor
Destructor stackobj ()
	_ClearStack
End Destructor

'Clears stack. 
Sub stackobj.ClearStack ()
	_ClearStack
End Sub

'Returns the number of items on the stack or 0 if empty.
Function stackobj.Count () As Integer
	Return _cnt
End Function

'Pushes an item onto the stack. Returns True if successful, False if an error occured.
Function stackobj.Push (dt As dataobj) As Integer
	Dim ret As Integer = TRUE
	
	'Set the default error code.
	_lasterror = stacknoerror
	'Allocate space for the new object.
	_stack = ReAllocate(_stack, SizeOf(dataobj) * (_cnt + 1))
	'Check for an error.
	If _stack <> NULL Then
		'Copy over the new data.
		_cnt += 1
		_stack[_cnt - 1] = dt
	Else
		ret = FALSE
		_lasterror = stacknomemory
	EndIf
	
	Return ret
End Function

'Pops an item from the stack. If stack is empty returns False, else True is returned.
Function stackobj.Pop (dt As dataobj) As Integer
	Dim ret As Integer = TRUE
	
	'Set the default error code.
	_lasterror = stacknoerror
	If (_cnt = 0) Or (_stack = NULL) Then
		ret = FALSE
		_lasterror = stackempty
	Else
		dt = _stack[_cnt - 1]
		_cnt -= 1
		If _cnt > 0 then
			_stack = ReAllocate(_stack, SizeOf(dataobj) * _cnt)
			If _stack = NULL Then
				ret = FALSE
				_lasterror = stacknomemory
			EndIf
		Else
			_ClearStack
		End If
	EndIf
	
	Return ret
	
End Function

'Returns the last error code.
Function stackobj.GetLastError () As stackerror
	Return _lasterror
End Function

'Returns the error as a string.
Function stackobj.GetErrorMessage (e As stackerror) As String
	Dim ret As String
	
	Select Case e
		Case stacknoerror
			ret = "Stack: No error"
		Case stackempty
			ret = "Stack: Stack empty"
		Case stacknomemory
			ret = "Stack: Stack allocation failed"
		Case Else
			ret = "Stack: Unknown error code"
	End Select
	
	Return ret
End Function

End Namespace
